#
# A module for reading dbf files. Tested ONLY on linux. 
#
# Author        : Michal Spalinski (mspal@fuw.edu.pl)
# Copying policy: unlimited (do what you want with it) 
# Warranty      : none whatsoever
#

"""
This is a module for reading dbf files. 

The code fragment

import dbf
db = dbf.dbf('mydata.dbf')

creates a dbf object db associated with an existing dbf file 'mydata.dbf'. 
Then:

-- db.fields returns a a list of tuples describing the fields
-- db.nrecs  returns the number of records
-- db[n] returns a tuple containing the contents of record number n (0 <= n < nrecs). 
   All field contents are represented as strings ...
-- db.status() prints some data about the dbf file

So, for example, to list the first two fields of all the records in mydata.dbf
one could try:

for r in db: 
    print r[0], r[1]

Good luck!
"""

from struct import unpack 
 
class dbf: 
    def __init__(self, fname): 
	self.f = open(fname,'r') 
	head = self.f.read(32) 
	if (head[0] != '\003') and (head[0] != '\203'): 
	    raise TypeError, 'Not a Dbase III+ file!' 
	(self.nrecs, self.hlen, self.rlen) = unpack('4xihh20x', head) 
	fdalen = (self.hlen - 33)/32 
	# read the field descriptor array 
	fda = [] 
	for k in range(fdalen): 
	    fda.append(self.f.read(32)) 
	# interpret the field descriptors 
	self.fields = [] 
	for fd in fda: 
	    bytes = unpack('12c4xbb14x', fd) 
	    field = '' 
	    for i in range(11): 
		if bytes[i] == '\000': 
		    break 
		field = field+bytes[i] 
	    type = bytes[11] 
	    length = bytes[12] 
	    dec = bytes[13] 
	    self.fields.append((field,type,length,dec)) 
 
    # record numbers go from 0 to self.nrecs-1
    def _get(self, recno): 
	offs = self.hlen + recno*self.rlen 
	self.f.seek(offs,0) 
	return self.f.read(self.rlen) 
     
    def __getitem__(self, recno):
	if recno < 0 or recno >= self.nrecs:
	    raise IndexError, 'No such record.'
	else:
	    raw = self._get(recno) 
	    res, pos = [], 0 
	    for field in self.fields: 
		end = pos+field[2] 
		item = raw[pos+1:end+1] 
		pos=end 
		res.append(item) 
	    return tuple(res) 

    def status(self):
	print ''
	print 'Header length     :', self.hlen
	print 'Record length     :', self.rlen
	print 'Number of records :',  self.nrecs
	print ''
	print '%-12s %-12s %-12s %-12s' % ('Field','Type','Length','Decimal')
	print '%-12s %-12s %-12s %-12s' % ('-----','----','------','-------')
	for k in self.fields:
	    print '%-12s %-12s %-12s %-12s' % k
	print ''

